@@ -1,40 +1,28 @@
 # Example demonstrating XOR with momentum backprop learning
 
 use strict;
-use AI::NNFlex::momentum;
+use AI::NNFlex::Backprop;
 use AI::NNFlex::Dataset;
 
 # Create the network 
 
-my $network = AI::NNFlex::momentum->new(randomconnections=>0,
-				randomweights=>1,
-				learningrate=>.1,
-				debug=>[],bias=>1,
+my $network = AI::NNFlex::Backprop->new(
+				learningrate=>.15,
+				bias=>1,
+				fahlmanconstant=>0.1,
 				momentum=>0.6,
 				round=>1);
 
 
 
 $network->add_layer(	nodes=>2,
-			persistentactivation=>0,
-			decay=>0.0,
-			randomactivation=>0,
-			threshold=>0.0,
 			activationfunction=>"tanh");
 
 
 $network->add_layer(	nodes=>2,
-			persistentactivation=>0,
-			decay=>0.0,
-			randomactivation=>0,
-			threshold=>0.0,
 			activationfunction=>"tanh");
 
 $network->add_layer(	nodes=>1,
-			persistentactivation=>0,
-			decay=>0.0,
-			randomactivation=>0,
-			threshold=>0.0,
 			activationfunction=>"linear");
 
 
